/*
 * The PyHST program is Copyright (C) 2002-2011 of the
 * European Synchrotron Radiation Facility (ESRF) and
 * Karlsruhe Institute of Technology (KIT).
 *
 * PyHST is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * hst is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
struct {
    cl_platform_id platform_id;
    cl_uint num_devices;
    cl_device_id device_ids[HST_MAX_OPENCL_DEVICES]; 
} g_hst_platform;

static HSTConstString hst_opencl_timers[] = { "complete", "data transfer", "filtering", "backprojection", NULL }; 


#ifdef PYHST_MEASURE_TIMINGS
inline void hst_measure_time(cl_ulong *timer, cl_command_queue queue, cl_event event)
{
    cl_ulong start, end;
    clFinish(queue);
    CL_CHECK_ERROR(clGetEventProfilingInfo(event, CL_PROFILING_COMMAND_END,
            sizeof(cl_ulong), &end, NULL));
    CL_CHECK_ERROR(clGetEventProfilingInfo(event, CL_PROFILING_COMMAND_START,
            sizeof(cl_ulong), &start, NULL));
    *timer += (end - start);
}
#else
#define hst_measure_time(timer, queue, event)
#endif

static HSTConstString *hst_opencl_get_timers(HSTReconstructorConstContextPtr rctx, double *timers)
{
#ifdef PYHST_MEASURE_TIMINGS
    assert(rctx);
    
    if (timers) {
        OpenCLContext *ctx = (OpenCLContext *) rctx;
        size_t timer_resolution;
        CL_CHECK_ERROR(clGetDeviceInfo(ctx->device_id,
                    CL_DEVICE_PROFILING_TIMER_RESOLUTION,       /* in nanoseconds */
                    sizeof(size_t), &timer_resolution, NULL));

        timers[1] = ctx->timer_transfer * 1.0e-9;
        timers[2] = ctx->timer_filter * 1.0e-9;
        timers[3] = ctx->timer_backprojection * 1.0e-9;
        timers[0] = timers[1] + timers[2] + timers[3];
    }
#endif

    return hst_opencl_timers;
}
